local Arathi = DBM:NewBossMod("Arathi", DBM_ARATHI, DBM_BGMOD_LANG["AB_DESCRIPTION"], DBM_OTHER, "Battlegrounds", 2);

Arathi.Version = "2.0";
Arathi.Author = "LeoLeal, Nitram, Tandanu";
Arathi.ExtraGUITab	= "BC Battlegrounds";

Arathi:RegisterEvents(
	"ZONE_CHANGED_NEW_AREA",
	"PLAYER_ENTERING_WORLD",
	"CHAT_MSG_BG_SYSTEM_HORDE",
	"CHAT_MSG_BG_SYSTEM_ALLIANCE",
	"CHAT_MSG_BG_SYSTEM_NEUTRAL",
	"CHAT_MSG_SPELL_PERIODIC_SELF_DAMAGE"
);
	

Arathi.ResPerSec = {
	[0] = 0,
	[1] = 10/12,
	[2] = 10/9,
	[3] = 10/6,
	[4] = 10/3,
	[5] = 30,
};

Arathi.LastTick = {
	LastAllyCount	= 0,
	LastHordeCount	= 0,
	Time = 0,
	Alliance = {
		Score	= 0,
		Time	= 0,
		Bases	= 0,
	},
	Horde = {
		Score	= 0,
		Time	= 0,
		Bases	= 0,
	},
};

Arathi.DropdownMenu = { --I don't use :AddOption() here, because I want to use the battleground boss mod's options table
	{
		variable = "DBM.AddOns.Battlegrounds.Options.ShowInviteTimer", 
		text = DBM_BGMOD_LANG.SHOW_INV_TIMER, 
		func = function() DBM.AddOns.Battlegrounds.Options.ShowInviteTimer = not DBM.AddOns.Battlegrounds.Options.ShowInviteTimer; end, 
	},
	
	{
		variable = "DBM.AddOns.Battlegrounds.Options.ColorByClass", 
		text = DBM_BGMOD_LANG.COLOR_BY_CLASS, 
		func = function() DBM.AddOns.Battlegrounds.Options.ColorByClass = not DBM.AddOns.Battlegrounds.Options.ColorByClass; end, 
	},
};

Arathi:AddOption("ShowAbFrame", true, DBM_BGMOD_LANG.AB_INFOFRAME_INFO, function()
	DBM:GetMod("Arathi").Options.ShowAbFrame = not DBM:GetMod("Arathi").Options.ShowAbFrame;
	if DBM:GetMod("Arathi").Options.ShowAbFrame and GetRealZoneText() == DBM_ARATHI then
		DBM:GetMod("Arathi"):ShowEstimatedPoints();
		DBM:GetMod("Arathi"):OnEvent("UpdateTimer");
	else
		DBM:GetMod("Arathi"):HideEstimatedPoints();
	end
end);


function Arathi:GetScore()
	if GetRealZoneText() == DBM_ARATHI then
		local _, _, mAllyInfo	= GetWorldStateUIInfo(1);
		local _, _, mHordeInfo	= GetWorldStateUIInfo(2);
		if not mAllyInfo or not mHordeInfo then
			return false;
		end

		local _, _, AllyBases, AllyScore	= string.find(mAllyInfo, DBM_BGMOD_LANG.AB_SCOREEXP);
		local _, _, HordeBases, HordeScore	= string.find(mHordeInfo, DBM_BGMOD_LANG.AB_SCOREEXP);
		AllyBases	= tonumber(AllyBases);
		AllyScore	= tonumber(AllyScore);
		HordeBases	= tonumber(HordeBases);
		HordeScore	= tonumber(HordeScore);
		if not AllyBases or not AllyScore or not HordeBases or not HordeScore then
			return false;
		end
	
		return true, AllyBases, AllyScore, HordeBases, HordeScore;
	else
		return false;
	end
end

function Arathi:ShowEstimatedPoints()
	if AlwaysUpFrame1Text and AlwaysUpFrame2Text then
		if not self.ScoreFrame1 then
			self.ScoreFrame1 = CreateFrame("Frame", nil, AlwaysUpFrame1);
			self.ScoreFrame1:SetHeight(10);
			self.ScoreFrame1:SetWidth(100);
			self.ScoreFrame1:SetPoint("LEFT", "AlwaysUpFrame1Text", "RIGHT", 4, 0)
			self.ScoreFrame1Text = self.ScoreFrame1:CreateFontString(nil, nil, "GameFontNormalSmall");
			self.ScoreFrame1Text:SetAllPoints(self.ScoreFrame1);
			self.ScoreFrame1Text:SetJustifyH("LEFT");
		end
		if not self.ScoreFrame2 then
			self.ScoreFrame2 = CreateFrame("Frame", nil, AlwaysUpFrame2);
			self.ScoreFrame2:SetHeight(10);
			self.ScoreFrame2:SetWidth(100);
			self.ScoreFrame2:SetPoint("LEFT", "AlwaysUpFrame2Text", "RIGHT", 4, 0)
			self.ScoreFrame2Text= self.ScoreFrame2:CreateFontString(nil, nil, "GameFontNormalSmall");
			self.ScoreFrame2Text:SetAllPoints(self.ScoreFrame2);
			self.ScoreFrame2Text:SetJustifyH("LEFT");
		end
		self.ScoreFrame1Text:SetText("");
		self.ScoreFrame1:Show();
		self.ScoreFrame2Text:SetText("");
		self.ScoreFrame2:Show();
	end
end

function Arathi:HideEstimatedPoints()
	if self.ScoreFrame1 and self.ScoreFrame2 then
		self.ScoreFrame1:Hide();
		self.ScoreFrame1Text:SetText("");
		self.ScoreFrame2:Hide();
		self.ScoreFrame2Text:SetText("");
	end
end

function Arathi:OnUpdate()
	if GetRealZoneText() == DBM_ARATHI then
		local mOk, AllyBases, AllyScore, HordeBases, HordeScore = self:GetScore();
		if not mOk then
			return;
		end
		
		self.LastTick.Alliance.Score = AllyScore;
		self.LastTick.Alliance.Bases = AllyBases;
		
		self.LastTick.Horde.Score = HordeScore;
		self.LastTick.Horde.Bases = HordeBases;
		
		self.LastTick.Time = GetTime(); --needed if the UpdateTimer event is not called by this function
		
		if self.LastTick.Alliance.Score < 2000 
		and self.LastTick.Horde.Score < 2000
		and (self.LastTick.Alliance.Score + self.LastTick.Horde.Score) > 0
		and (self.LastTick.Alliance.Bases + self.LastTick.Horde.Bases) > 0
		and (self.LastTick.LastAllyCount ~= self.LastTick.Alliance.Bases or self.LastTick.LastHordeCount ~= self.LastTick.Horde.Bases) then
			self:OnEvent("UpdateTimer");
		end
	else --not in Arathi, end timers
		self:EndStatusBarTimer("AB_WINALLY", true);
		self:EndStatusBarTimer("AB_WINHORDE", true);
	end
end

function Arathi:OnEvent(event)
	if (event == "ZONE_CHANGED_NEW_AREA" or event == "PLAYER_ENTERING_WORLD") and GetRealZoneText() == DBM_ARATHI and self.Options.ShowAbFrame then
		self:ShowEstimatedPoints();
		
	elseif (event == "ZONE_CHANGED_NEW_AREA" or event == "PLAYER_ENTERING_WORLD") and GetRealZoneText() ~= DBM_ARATHI then
		self:HideEstimatedPoints();
		
	elseif event == "CHAT_MSG_SPELL_PERIODIC_SELF_DAMAGE" then --speed boots
		if arg1 and string.find(arg1, DBM_BGMOD_LANG.WSG_BOOTS_EXPR) then
			self:StartStatusBarTimer(10, "Speed Boots", "Interface\\Icons\\Spell_Fire_BurningSpeed", true); --respawn time varies, ~3 minutes
		end
	
	elseif event == "CHAT_MSG_BG_SYSTEM_NEUTRAL" then --"game starts in.." timers
		if arg1 == DBM_BGMOD_LANG.AB_START60SEC then
			self:StartStatusBarTimer(62, "Begins");
		elseif arg1 == DBM_BGMOD_LANG.AB_START30SEC then		
			if not self:GetStatusBarTimerTimeLeft("Begins") then
				self:StartStatusBarTimer(62, "Begins");
			end
			self:UpdateStatusBarTimer("Begins", 31, 62);
		end
	
	elseif event == "UpdateTimer" then
		local AllyTime = ((2000 - self.LastTick.Alliance.Score) / self.ResPerSec[self.LastTick.Alliance.Bases]) - (GetTime() - self.LastTick.Time); -- possible division through zero, but Lua does not complain about this...1/0 = 1.#INF
		local HordeTime = ((2000 - self.LastTick.Horde.Score) / self.ResPerSec[self.LastTick.Horde.Bases]) - (GetTime() - self.LastTick.Time);
		if AllyTime > 5000 then		AllyTime = 5000; end
		if HordeTime > 5000 then	HordeTime = 5000; end
		
		self.LastTick.LastAllyCount	= self.LastTick.Alliance.Bases; 
		self.LastTick.LastHordeCount = self.LastTick.Horde.Bases;
		
		if self:GetStatusBarTimerTimeLeft("AB_WINALLY") and self:GetStatusBarTimerTimeLeft("AB_WINHORDE") then -- should never happen
			self:EndStatusBarTimer("AB_WINALLY", true);
			self:EndStatusBarTimer("AB_WINHORDE", true);
		end		
		
		if AllyTime == HordeTime then -- draw
			self:EndStatusBarTimer("AB_WINALLY", true);
			self:EndStatusBarTimer("AB_WINHORDE", true);
			self.ScoreFrame1Text:SetText("");
			self.ScoreFrame2Text:SetText("");
			
		elseif AllyTime > HordeTime then -- Horde wins
			if self.ScoreFrame1Text and self.ScoreFrame2Text then
				local AllyPoints = math.floor(math.floor(((HordeTime * self.ResPerSec[self.LastTick.Alliance.Bases]) + self.LastTick.Alliance.Score) / 10) * 10);
				self.ScoreFrame1Text:SetText("("..AllyPoints..")");
				self.ScoreFrame2Text:SetText("(2000)");
			end
			
			if self:GetStatusBarTimerTimeLeft("AB_WINALLY") then
				local timeLeft, timeElapsed = self:GetStatusBarTimerTimeLeft("AB_WINALLY");
				self:UpdateStatusBarTimer("AB_WINALLY", nil, (timeElapsed + HordeTime), "AB_WINHORDE", "Interface\\Icons\\INV_BannerPVP_01.blp", true);
			elseif( self:GetStatusBarTimerTimeLeft("AB_WINHORDE") ) then
				local timeLeft, timeElapsed = self:GetStatusBarTimerTimeLeft("AB_WINHORDE");
				self:UpdateStatusBarTimer("AB_WINHORDE", nil, (timeElapsed + HordeTime), nil, nil, true);
			else
				self:StartStatusBarTimer(HordeTime, "AB_WINHORDE", "Interface\\Icons\\INV_BannerPVP_01.blp", true);
			end
			
		elseif HordeTime > AllyTime then -- Alliance wins
			if self.ScoreFrame1Text and self.ScoreFrame2Text then
				local HordePoints = math.floor(math.floor(((AllyTime * self.ResPerSec[self.LastTick.Horde.Bases]) + self.LastTick.Horde.Score) / 10) * 10);
				self.ScoreFrame2Text:SetText("("..HordePoints..")");
				self.ScoreFrame1Text:SetText("(2000)");						
			end
			
			if( self:GetStatusBarTimerTimeLeft("AB_WINHORDE") ) then
				local timeLeft, timeElapsed = self:GetStatusBarTimerTimeLeft("AB_WINHORDE");
				self:UpdateStatusBarTimer("AB_WINHORDE", nil, (timeElapsed + AllyTime), "AB_WINALLY", "Interface\\Icons\\INV_BannerPVP_02.blp", true);
			
			elseif( self:GetStatusBarTimerTimeLeft("AB_WINALLY") ) then
				local timeLeft, timeElapsed = self:GetStatusBarTimerTimeLeft("AB_WINALLY");
				self:UpdateStatusBarTimer("AB_WINALLY", nil, (timeElapsed + AllyTime), nil, nil, true);
			else
				self:StartStatusBarTimer(AllyTime, "AB_WINALLY", "Interface\\Icons\\INV_BannerPVP_02.blp", true);
			end
		end
		
	elseif event == "CHAT_MSG_BG_SYSTEM_ALLIANCE" then
		for index, value in ipairs(DBM_BGMOD_LANG.AB_TARGETS) do
			if string.find(arg1, value) then
				if string.find(arg1, DBM_BGMOD_LANG.AB_HASASSAULTED) or string.find(arg1, DBM_BGMOD_LANG.AB_CLAIMSTHE) then
					self:EndStatusBarTimer("Horde: "..DBM.Capitalize(DBM_BGMOD_EN_TARGET_AB[index])); -- we still use the horde/ally prefix, because we don't want to create duplicated timers for old clients
					self:StartStatusBarTimer(64,  "Alliance: "..DBM.Capitalize(DBM_BGMOD_EN_TARGET_AB[index]), "Interface\\Icons\\INV_BannerPVP_02.blp", nil, nil, 0, 0, 1);
					
				elseif string.find(arg1, DBM_BGMOD_LANG.AB_HASDEFENDEDTHE) or string.find(arg1, DBM_BGMOD_LANG.AB_HASTAKENTHE) then
					if string.find(arg1, DBM_BGMOD_LANG.AB_HASTAKENTHE) then
						if DBM_BGMOD_LANG.AB_TARGETS_ANNOUNCE and DBM_BGMOD_LANG.AB_TARGETS_ANNOUNCE[index] then
							self:Announce(string.format(DBM_BGMOD_LANG.ALLI_TAKE_ANNOUNCE, DBM_BGMOD_LANG.AB_TARGETS_ANNOUNCE[index]));
						else
							self:Announce(string.format(DBM_BGMOD_LANG.ALLI_TAKE_ANNOUNCE, DBM.Capitalize(DBM_BGMOD_LANG.AB_TARGETS[index])));
						end
						local oldFlashColor = DBM.Options.FlashColor;
						DBM.Options.FlashColor = "blue";
						self:AddSpecialWarning("", true);
						DBM.Options.FlashColor = oldFlashColor;
					end
					self:EndStatusBarTimer("Alliance: "..DBM.Capitalize(DBM_BGMOD_EN_TARGET_AB[index]), true);
					self:EndStatusBarTimer("Horde: "..DBM.Capitalize(DBM_BGMOD_EN_TARGET_AB[index]), true);
				end
			end
		end
		
	elseif event == "CHAT_MSG_BG_SYSTEM_HORDE" then
		for index, value in ipairs(DBM_BGMOD_LANG.AB_TARGETS) do
			if string.find(arg1, value) then
				if string.find(arg1, DBM_BGMOD_LANG.AB_HASASSAULTED) or string.find(arg1, DBM_BGMOD_LANG.AB_CLAIMSTHE) then
					self:EndStatusBarTimer("Alliance: "..DBM.Capitalize(DBM_BGMOD_EN_TARGET_AB[index]));
					self:StartStatusBarTimer(64,  "Horde: "..DBM.Capitalize(DBM_BGMOD_EN_TARGET_AB[index]), "Interface\\Icons\\INV_BannerPVP_01.blp", nil, nil, 1, 0, 0);
					
				elseif string.find(arg1, DBM_BGMOD_LANG.AB_HASDEFENDEDTHE) or string.find(arg1, DBM_BGMOD_LANG.AB_HASTAKENTHE) then
					if string.find(arg1, DBM_BGMOD_LANG.AB_HASTAKENTHE) then
						if DBM_BGMOD_LANG.AB_TARGETS_ANNOUNCE and DBM_BGMOD_LANG.AB_TARGETS_ANNOUNCE[index] then
							self:Announce(string.format(DBM_BGMOD_LANG.HORDE_TAKE_ANNOUNCE, DBM_BGMOD_LANG.AB_TARGETS_ANNOUNCE[index]));
						else
							self:Announce(string.format(DBM_BGMOD_LANG.HORDE_TAKE_ANNOUNCE, DBM.Capitalize(DBM_BGMOD_LANG.AB_TARGETS[index])));
						end
						local oldFlashColor = DBM.Options.FlashColor;
						DBM.Options.FlashColor = "red";
						self:AddSpecialWarning("", true);
						DBM.Options.FlashColor = oldFlashColor;
					end
					self:EndStatusBarTimer("Alliance: "..DBM.Capitalize(DBM_BGMOD_EN_TARGET_AB[index]), true);
					self:EndStatusBarTimer("Horde: "..DBM.Capitalize(DBM_BGMOD_EN_TARGET_AB[index]), true);
				end
			end
		end
	end
end